@@ -81,7 +81,7 @@ PetscErrorCode SVDSetOperators(SVD svd,Mat A,Mat B)
 /*@
    SVDGetOperators - Get the matrices associated with the singular value problem.
 
-   Not Collective
+   Collective
 
    Input Parameter:
 .  svd - the singular value solver context
@@ -136,13 +136,15 @@ PetscErrorCode SVDSetSignature(SVD svd,Vec omega)
     PetscCall(VecGetLocalSize(omega,&n));
     PetscCall(MatGetSize(svd->OP,&Ma,NULL));
     PetscCall(MatGetLocalSize(svd->OP,&ma,NULL));
-    PetscCheck(N==Ma,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONG,"Global size of signature (%" PetscInt_FMT ") does not match the row size of A (%" PetscInt_FMT ")",N,Ma);
-    PetscCheck(n==ma,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONG,"Local size of signature (%" PetscInt_FMT ") does not match the local row size of A (%" PetscInt_FMT ")",n,ma);
+    PetscCheck(N==Ma,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_SIZ,"Global size of signature (%" PetscInt_FMT ") does not match the row size of A (%" PetscInt_FMT ")",N,Ma);
+    PetscCheck(n==ma,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_SIZ,"Local size of signature (%" PetscInt_FMT ") does not match the local row size of A (%" PetscInt_FMT ")",n,ma);
   }
 
-  if (omega) PetscCall(PetscObjectReference((PetscObject)omega));
   PetscCall(VecDestroy(&svd->omega));
-  svd->omega = omega;
+  if (omega) {
+    PetscCall(VecDuplicate(omega,&svd->omega));
+    PetscCall(VecCopy(omega,svd->omega));
+  }
   svd->state = SVD_STATE_INITIAL;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -158,16 +160,23 @@ PetscErrorCode SVDSetSignature(SVD svd,Vec omega)
    Output Parameter:
 .  omega - a vector containing the diagonal elements of the signature matrix
 
+   Notes:
+   The signature matrix is relevant only for hyperbolic problems (HSVD).
+   If no signature has been set, this function will return a vector of all ones.
+
+   The user should pass a previously created Vec with the appropriate dimension.
+
    Level: intermediate
 
 .seealso: SVDSetSignature()
 @*/
-PetscErrorCode SVDGetSignature(SVD svd,Vec *omega)
+PetscErrorCode SVDGetSignature(SVD svd,Vec omega)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscAssertPointer(omega,2);
-  *omega = svd->omega;
+  PetscValidHeaderSpecific(omega,VEC_CLASSID,2);
+  if (svd->omega) PetscCall(VecCopy(svd->omega,omega));
+  else PetscCall(VecSet(omega,1.0));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -217,10 +226,11 @@ PetscErrorCode SVDSetDSType(SVD svd)
 PetscErrorCode SVDSetUp(SVD svd)
 {
   PetscBool      flg;
-  PetscInt       M,N,P=0,k,maxnsol;
+  PetscInt       M,N,P=0,k,maxnsol,m,Nom,nom;
   SlepcSC        sc;
   Vec            *T;
   BV             bv;
+  SVDStoppingCtx ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
@@ -233,7 +243,6 @@ PetscErrorCode SVDSetUp(SVD svd)
   /* set default solver type (SVDSetFromOptions was not called) */
   if (!((PetscObject)svd)->type_name) PetscCall(SVDSetType(svd,SVDCROSS));
   if (!svd->ds) PetscCall(SVDGetDS(svd,&svd->ds));
-  PetscCall(SVDSetDSType(svd));
 
   /* check matrices */
   PetscCheck(svd->OP,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_WRONGSTATE,"SVDSetOperators() must be called first");
@@ -257,6 +266,9 @@ PetscErrorCode SVDSetUp(SVD svd)
     }
   }
 
+  /* set DS type once the default problem type has been determined */
+  PetscCall(SVDSetDSType(svd));
+
   /* determine how to handle the transpose */
   svd->expltrans = PETSC_TRUE;
   if (svd->impltrans) svd->expltrans = PETSC_FALSE;
@@ -271,9 +283,16 @@ PetscErrorCode SVDSetUp(SVD svd)
 
   /* get matrix dimensions */
   PetscCall(MatGetSize(svd->OP,&M,&N));
+  if (M == 0 || N == 0) PetscFunctionReturn(PETSC_SUCCESS);
   if (svd->isgeneralized) {
     PetscCall(MatGetSize(svd->OPb,&P,NULL));
     PetscCheck(M+P>=N,PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"The case when [A;B] has less rows than columns is not supported");
+  } else if (svd->ishyperbolic) {
+    PetscCall(VecGetSize(svd->omega,&Nom));
+    PetscCall(VecGetLocalSize(svd->omega,&nom));
+    PetscCall(MatGetLocalSize(svd->OP,&m,NULL));
+    PetscCheck(Nom==M,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_SIZ,"Global size of signature (%" PetscInt_FMT ") does not match the row size of A (%" PetscInt_FMT ")",Nom,M);
+    PetscCheck(nom==m,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_SIZ,"Local size of signature (%" PetscInt_FMT ") does not match the local row size of A (%" PetscInt_FMT ")",nom,m);
   }
 
   /* build transpose matrix */
@@ -328,20 +347,31 @@ PetscErrorCode SVDSetUp(SVD svd)
   maxnsol = svd->isgeneralized? PetscMin(PetscMin(M,N),P): PetscMin(M,N);
   svd->ncv = PetscMin(svd->ncv,maxnsol);
   svd->nsv = PetscMin(svd->nsv,maxnsol);
-  PetscCheck(svd->ncv==PETSC_DEFAULT || svd->nsv<=svd->ncv,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"nsv bigger than ncv");
+  PetscCheck(svd->ncv==PETSC_DETERMINE || svd->nsv<=svd->ncv,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"nsv bigger than ncv");
 
   /* relative convergence criterion is not allowed in GSVD */
   if (svd->conv==(SVDConv)-1) PetscCall(SVDSetConvergenceTest(svd,svd->isgeneralized?SVD_CONV_NORM:SVD_CONV_REL));
   PetscCheck(!svd->isgeneralized || svd->conv!=SVD_CONV_REL,PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Relative convergence criterion is not allowed in GSVD");
 
-  /* initialization of matrix norm (stardard case only, for GSVD it is done inside setup()) */
+  /* initialization of matrix norm (standard case only, for GSVD it is done inside setup()) */
   if (!svd->isgeneralized && svd->conv==SVD_CONV_NORM && !svd->nrma) PetscCall(MatNorm(svd->OP,NORM_INFINITY,&svd->nrma));
 
+  /* threshold stopping test */
+  if (svd->stop==SVD_STOP_THRESHOLD) {
+    PetscCheck(svd->thres>0.0,PetscObjectComm((PetscObject)svd),PETSC_ERR_USER_INPUT,"Must give a threshold value with SVDSetThreshold()");
+    PetscCheck(svd->which==SVD_LARGEST || !svd->threlative,PetscObjectComm((PetscObject)svd),PETSC_ERR_USER_INPUT,"Can only use a relative threshold when which=SVD_LARGEST");
+    PetscCall(PetscNew(&ctx));
+    ctx->thres      = svd->thres;
+    ctx->threlative = svd->threlative;
+    ctx->which      = svd->which;
+    PetscCall(SVDSetStoppingTestFunction(svd,SVDStoppingThreshold,ctx,PetscCtxDestroyDefault));
+  }
+
   /* call specific solver setup */
   PetscUseTypeMethod(svd,setup);
 
   /* set tolerance if not yet set */
-  if (svd->tol==(PetscReal)PETSC_DEFAULT) svd->tol = SLEPC_DEFAULT_TOL;
+  if (svd->tol==(PetscReal)PETSC_DETERMINE) svd->tol = SLEPC_DEFAULT_TOL;
 
   /* fill sorting criterion context */
   PetscCall(DSGetSlepcSC(svd->ds,&sc));
@@ -374,7 +404,7 @@ PetscErrorCode SVDSetUp(SVD svd)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    SVDSetInitialSpaces - Specify two basis of vectors that constitute the initial
    right and/or left spaces.
 
@@ -445,9 +475,10 @@ PetscErrorCode SVDSetDimensions_Default(SVD svd)
     PetscCall(MatGetSize(svd->OPb,&P,NULL));
     maxnsol = PetscMin(maxnsol,P);
   }
-  if (svd->ncv!=PETSC_DEFAULT) { /* ncv set */
+  if (svd->nsv==0 && svd->stop!=SVD_STOP_THRESHOLD) svd->nsv = 1;
+  if (svd->ncv!=PETSC_DETERMINE) { /* ncv set */
     PetscCheck(svd->ncv>=svd->nsv,PetscObjectComm((PetscObject)svd),PETSC_ERR_USER_INPUT,"The value of ncv must be at least nsv");
-  } else if (svd->mpd!=PETSC_DEFAULT) { /* mpd set */
+  } else if (svd->mpd!=PETSC_DETERMINE) { /* mpd set */
     svd->ncv = PetscMin(maxnsol,svd->nsv+svd->mpd);
   } else { /* neither set: defaults depend on nsv being small or large */
     if (svd->nsv<500) svd->ncv = PetscMin(maxnsol,PetscMax(2*svd->nsv,10));
@@ -456,7 +487,7 @@ PetscErrorCode SVDSetDimensions_Default(SVD svd)
       svd->ncv = PetscMin(maxnsol,svd->nsv+svd->mpd);
     }
   }
-  if (svd->mpd==PETSC_DEFAULT) svd->mpd = svd->ncv;
+  if (svd->mpd==PETSC_DETERMINE) svd->mpd = svd->ncv;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -467,7 +498,7 @@ PetscErrorCode SVDSetDimensions_Default(SVD svd)
    Collective
 
    Input Parameters:
-+  svd   - eigensolver context
++  svd   - singular value solver context
 -  extra - number of additional positions, used for methods that require a
            working basis slightly larger than ncv
 
@@ -502,7 +533,7 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
   /* allocate V */
   if (!svd->V) PetscCall(SVDGetBV(svd,&svd->V,NULL));
   if (!oldsize) {
-    if (!((PetscObject)(svd->V))->type_name) PetscCall(BVSetType(svd->V,BVMAT));
+    if (!((PetscObject)svd->V)->type_name) PetscCall(BVSetType(svd->V,BVMAT));
     PetscCall(MatCreateVecsEmpty(svd->A,&tr,NULL));
     PetscCall(BVSetSizesFromVec(svd->V,tr,requested));
     PetscCall(VecDestroy(&tr));
@@ -511,7 +542,7 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
   if (svd->leftbasis && !svd->isgeneralized) {
     if (!svd->U) PetscCall(SVDGetBV(svd,NULL,&svd->U));
     if (!oldsize) {
-      if (!((PetscObject)(svd->U))->type_name) PetscCall(BVSetType(svd->U,((PetscObject)(svd->V))->type_name));
+      if (!((PetscObject)svd->U)->type_name) PetscCall(BVSetType(svd->U,((PetscObject)svd->V)->type_name));
       PetscCall(MatCreateVecsEmpty(svd->A,NULL,&tl));
       PetscCall(BVSetSizesFromVec(svd->U,tl,requested));
       PetscCall(VecDestroy(&tl));
@@ -519,7 +550,7 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
   } else if (svd->isgeneralized) {  /* left basis for the GSVD */
     if (!svd->U) PetscCall(SVDGetBV(svd,NULL,&svd->U));
     if (!oldsize) {
-      if (!((PetscObject)(svd->U))->type_name) PetscCall(BVSetType(svd->U,((PetscObject)(svd->V))->type_name));
+      if (!((PetscObject)svd->U)->type_name) PetscCall(BVSetType(svd->U,((PetscObject)svd->V)->type_name));
       PetscCall(SVDCreateLeftTemplate(svd,&tl));
       PetscCall(BVSetSizesFromVec(svd->U,tl,requested));
       PetscCall(VecDestroy(&tl));
@@ -527,3 +558,55 @@ PetscErrorCode SVDAllocateSolution(SVD svd,PetscInt extra)
   }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
+
+/*@
+   SVDReallocateSolution - Reallocate memory storage for common variables such
+   as the singular values and the basis vectors.
+
+   Collective
+
+   Input Parameters:
++  svd     - singular value solver context
+-  newsize - new size
+
+   Developer Notes:
+   This is SLEPC_EXTERN because it may be required by user plugin SVD
+   implementations.
+
+   This is called during the iteration in case the threshold stopping test has
+   been selected.
+
+   Level: developer
+
+.seealso: SVDAllocateSolution(), SVDSetThreshold()
+@*/
+PetscErrorCode SVDReallocateSolution(SVD svd,PetscInt newsize)
+{
+  PetscInt  oldsize,*nperm;
+  PetscReal *nsigma,*nerrest,*nsign;
+
+  PetscFunctionBegin;
+  PetscCall(BVGetSizes(svd->V,NULL,NULL,&oldsize));
+  if (oldsize>=newsize) PetscFunctionReturn(PETSC_SUCCESS);
+  PetscCall(PetscInfo(svd,"Reallocating basis vectors to %" PetscInt_FMT " columns\n",newsize));
+
+  /* reallocate sigma */
+  PetscCall(PetscMalloc3(newsize,&nsigma,newsize,&nperm,newsize,&nerrest));
+  PetscCall(PetscArraycpy(nsigma,svd->sigma,oldsize));
+  PetscCall(PetscArraycpy(nperm,svd->perm,oldsize));
+  PetscCall(PetscArraycpy(nerrest,svd->errest,oldsize));
+  PetscCall(PetscFree3(svd->sigma,svd->perm,svd->errest));
+  svd->sigma  = nsigma;
+  svd->perm   = nperm;
+  svd->errest = nerrest;
+  if (svd->ishyperbolic) {
+    PetscCall(PetscMalloc1(newsize,&nsign));
+    PetscCall(PetscArraycpy(nsign,svd->sign,oldsize));
+    PetscCall(PetscFree(svd->sign));
+    svd->sign = nsign;
+  }
+  /* reallocate V,U */
+  PetscCall(BVResize(svd->V,newsize,PETSC_TRUE));
+  if (svd->leftbasis || svd->isgeneralized) PetscCall(BVResize(svd->U,newsize,PETSC_TRUE));
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
